
subroutine CalcEnergy(Pos, NAtom, L, rc2, PEnergy, Dim, Nmax)
    implicit none
    integer, intent(in) :: NAtom, Dim, Nmax
    real(8), intent(in), dimension(0:Nmax-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc2
    real(8), intent(out) :: PEnergy
    real(8), dimension(Dim) :: rijx, Posi
    real(8) :: rij, rij2, irij2, irij6, irij12
    real(8) :: PEc, irc2, irc6, irc12
    integer :: i, j
    PEnergy = 0.

    !shift energy at cutoff
    irc2 = 1. / rc2
    irc6 = irc2 * irc2 * irc2
    irc12 = irc6 * irc6
    PEc = 4. * (irc12 - irc6)

    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            rijx = Pos(j,:) - Posi
            rijx = rijx - L * dnint(rijx / L)
            rij2 = sum(rijx * rijx)
            if (rij2 < rc2) then
                !particles are within the cut-off
                irij2 = 1. / rij2              !inverse squared distance
                irij6 = irij2 * irij2 * irij2  !inverse sixth distance
                irij12 = irij6 * irij6         !inverse twelvth distance
                PEnergy = PEnergy + 4. * (irij12 - irij6) - PEc
            endif
        enddo
    enddo
end subroutine


subroutine CalcParticleEnergy(P, Pos, NAtom, L, rc2, PEnergy, Dim, Nmax)
    implicit none
    integer, intent(in) :: P, Dim, NAtom, Nmax
    real(8), intent(in), dimension(0:Nmax-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc2
    real(8), intent(out) :: PEnergy
    real(8), dimension(Dim) :: rijx, Posi
    real(8) :: rij, rij2, irij2, irij6, irij12
    real(8) :: PEc, irc2, irc6, irc12
    integer :: j
    PEnergy = 0.

    !shift energy at cutoff
    irc2 = 1. / rc2
    irc6 = irc2 * irc2 * irc2
    irc12 = irc6 * irc6
    PEc = 4. * (irc12 - irc6)

    !store Pos(P,:) in a temporary array for faster access in j loop
    Posi = Pos(P,:)

    do j = 0, NAtom - 1
        if (j == P) then
            cycle
        endif
        rijx = Pos(j,:) - Posi
        rijx = rijx - L * dnint(rijx / L)
        rij2 = sum(rijx * rijx)
        if (rij2 < rc2) then
            !particles are within the cut-off
            irij2 = 1. / rij2              !inverse squared distance
            irij6 = irij2 * irij2 * irij2  !inverse sixth distance
            irij12 = irij6 * irij6         !inverse twelvth distance
            PEnergy = PEnergy + 4. * (irij12 - irij6) - PEc
        endif
    enddo
end subroutine
